#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoComplexity.h>

//
// Kevin Beason
// 4/4/00
//

float PresentTime = 0.0;
float DTime = 0.01;
SoXtExaminerViewer *myViewer;

SoSeparator *
makeScene ()
  {
  SoSeparator *root = new SoSeparator();
  srand48(0);

  SoShapeHints *twoSided = new SoShapeHints();
  twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
  twoSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  root->addChild(twoSided);

  SoComplexity *myComplexity = new SoComplexity();
  myComplexity->textureQuality.setValue(1.0);
  root->addChild( myComplexity );

  // create box
  static float vertexPositions[20][3] =
  { //  x     y     z
    {  -1.0,  -1.0, -1.0 },  // 0      4 -- 6    
    {   1.0,  -1.0, -1.0 },  // 1    / |  / |
    {  -1.0,   1.0, -1.0 },  // 2  5 -- 7   |
    {   1.0,   1.0, -1.0 },  // 3  |   ||   |
    {  -1.0,  -1.0,  1.0 },  // 4  |   0|-- 2
    {   1.0,  -1.0,  1.0 },  // 5  | /  | /
    {  -1.0,   1.0,  1.0 },  // 6  1 -- 3
    {   1.0,   1.0,  1.0 },  // 7
    {  0.33, -0.67,  1.0 },  // 8      
    { -0.33, -0.67,  1.0 },  // 9 
    {  0.33, -0.33,  1.0 },  // 10
    { -0.33, -0.33,  1.0 },  // 11        4--18-19--6
    {   1.0,  -0.5,  1.0 },  // 12       /  /   /  /
    {   1.0,  -0.1,  1.0 },  // 13      16-9--11-17      Top Hole
    {  0.33,  -1.0,  1.0 },  // 14     /  /   /  /
    {  0.33,   1.0,  1.0 },  // 15    14-8--10-15
    { -0.33,  -1.0,  1.0 },  // 16   /  /   /  /
    { -0.33,   1.0,  1.0 },  // 17  5--12-13--7
    {  -1.0,  -0.5,  1.0 },  // 18
    {  -1.0,  -0.1,  1.0 }   // 19
  };

  SoSeparator *boxSep = new SoSeparator();
  root->addChild(boxSep);

  // static int indicesTop[5]    = { 5, 4, 6, 7, -1 };
  static int indicesLight[5] = { 8, 9, 11, 10, -1 };
  static int indicesTop[20] =
  {
    16,  4,  6, 17, -1,
    14, 16,  9,  8, -1,
    10, 11, 17, 15, -1,
     5, 14, 15 , 7, -1
  };
  static int indicesBottom[5] = { 0, 1, 3, 2, -1 };
  static int indicesLeft[5]   = { 0, 4, 5, 1, -1 };
  static int indicesRight[5]  = { 3, 7, 6, 2, -1 };
  static int indicesFront[5]  = { 1, 5, 7, 3, -1 };
  static int indicesBack[5]   = { 2, 6, 4, 0, -1 };

  SoIndexedFaceSet *lightFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *topFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *bottomFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *leftFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *rightFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *frontFaceSet = new SoIndexedFaceSet;
  SoIndexedFaceSet *backFaceSet = new SoIndexedFaceSet;

  SoCoordinate3 *boxCoords = new SoCoordinate3;
  boxCoords->point.setValues(0, 20, vertexPositions);
  boxSep->addChild(boxCoords);

  SoSeparator *lightSep = new SoSeparator();
  SoSeparator *topSep = new SoSeparator();
  SoSeparator *bottomSep = new SoSeparator();
  SoSeparator *leftSep = new SoSeparator();
  SoSeparator *rightSep = new SoSeparator();
  SoSeparator *frontSep = new SoSeparator();
  SoSeparator *backSep = new SoSeparator();

  boxSep->addChild(lightSep);
  boxSep->addChild(topSep);
  boxSep->addChild(bottomSep);
  boxSep->addChild(leftSep);
  boxSep->addChild(rightSep);
  boxSep->addChild(frontSep);
  boxSep->addChild(backSep);

  lightFaceSet->coordIndex.setValues(0, 5, indicesLight);
  topFaceSet->coordIndex.setValues(0, 20, indicesTop);
  bottomFaceSet->coordIndex.setValues(0, 5, indicesBottom);
  leftFaceSet->coordIndex.setValues(0, 5, indicesLeft);
  rightFaceSet->coordIndex.setValues(0, 5, indicesRight);
  frontFaceSet->coordIndex.setValues(0, 5, indicesFront);
  backFaceSet->coordIndex.setValues(0, 5, indicesBack);

  // turn on back face culling  
  SoShapeHints *topShapeHints = new SoShapeHints();
  topShapeHints->vertexOrdering.setValue(SoShapeHints::COUNTERCLOCKWISE);
  topShapeHints->shapeType.setValue(SoShapeHints::SOLID);
  lightSep->addChild( topShapeHints );
  topSep->addChild( topShapeHints );
  bottomSep->addChild( topShapeHints );
  leftSep->addChild( topShapeHints );
  rightSep->addChild( topShapeHints );
  frontSep->addChild( topShapeHints );
  backSep->addChild( topShapeHints );

  SoMaterial *lightMaterial = new SoMaterial;
  SoMaterial *topMaterial = new SoMaterial;  
  SoMaterial *bottomMaterial = new SoMaterial;  
  SoMaterial *leftMaterial = new SoMaterial;  
  SoMaterial *rightMaterial = new SoMaterial;  
  SoMaterial *frontMaterial = new SoMaterial;  
  SoMaterial *backMaterial = new SoMaterial;  

  lightMaterial->diffuseColor.setValue(1.0, 1.0, 1.0);
  lightMaterial->emissiveColor.setValue(1.0, 1.0, 1.0);
  lightMaterial->shininess.setValue(0.0);
    
  topMaterial   ->diffuseColor.setValue(1.0, 1.0, 0.5);
  bottomMaterial->diffuseColor.setValue(1.0, 1.0, 0.5);
  bottomMaterial->shininess.setValue(0.5);
  leftMaterial  ->diffuseColor.setValue(0.0, 1.0, 0.0);
  rightMaterial ->diffuseColor.setValue(1.0, 0.0, 0.0);
  frontMaterial ->diffuseColor.setValue(0.0, 0.0, 1.0);
  backMaterial  ->diffuseColor.setValue(1.0, 1.0, 1.0);

  lightSep->addChild(lightMaterial);
  topSep->addChild(topMaterial);
  bottomSep->addChild(bottomMaterial);
  //leftSep->addChild(leftMaterial);
  rightSep->addChild(rightMaterial);
  //frontSep->addChild(frontMaterial);
  backSep->addChild(backMaterial);

  // textures
  SoTexture2 *frontTexture = new SoTexture2();
  frontTexture->filename.setValue("/home/beason/photo/proj04/scene/front3.blur19.gif");
  frontSep->addChild(frontTexture);
  //SoTexture2 *leftTexture = new SoTexture2();
  //leftTexture->filename.setValue("left3.blur19.gif");
  //leftSep->addChild(leftTexture);
  
  lightSep->addChild(lightFaceSet);
  topSep->addChild(topFaceSet);
  bottomSep->addChild(bottomFaceSet);
  leftSep->addChild(leftFaceSet);
  rightSep->addChild(rightFaceSet);
  frontSep->addChild(frontFaceSet);
  backSep->addChild(backFaceSet);

  // create pyramid
  static float pyramidPositions[4][3] =
  {
    {  0.0,    0.0, 0.71 }, // 0  2-----3
    {  1.0,    0.0,  0.0 }, // 1   \\0//
    { -0.5,  -0.87,  0.0 }, // 2    \|/
    { -0.5,   0.87,  0.0 }  // 3     1
  };

  static int indicesPyramid[16] =
  {
    1, 0, 3, -1,
    2, 0, 1, -1,
    3, 0, 2, -1,
    1, 2, 3, -1
  };

  SoSeparator *pyramidSep = new SoSeparator();
  root->addChild(pyramidSep);

  SoTransform *pyramidTrans = new SoTransform();
  pyramidTrans->translation.setValue(-0.5, -0.5, -1.0);
  pyramidTrans->scaleFactor.setValue(0.5, 0.5, 0.5);
  pyramidTrans->rotation.setValue(0, 0, 1, 0.8);
  pyramidSep->addChild(pyramidTrans);

  SoMaterial *pyramidMaterial = new SoMaterial();
  pyramidMaterial->diffuseColor.setValue(1.0, 0.0, 0.0);
  pyramidSep->addChild(pyramidMaterial);
  
  SoCoordinate3 *pyramidCoords = new SoCoordinate3;
  pyramidCoords->point.setValues(0, 4, pyramidPositions);
  pyramidSep->addChild(pyramidCoords);

  SoIndexedFaceSet *pyramidFaceSet = new SoIndexedFaceSet;
  pyramidFaceSet->coordIndex.setValues(0, 16, indicesPyramid);
  pyramidSep->addChild(pyramidFaceSet);

  // create box
  SoSeparator *cubeSep = new SoSeparator();
  root->addChild(cubeSep);

  SoTransform *cubeTrans = new SoTransform();
  cubeTrans->translation.setValue(0.4, 0.0, -0.75);
  cubeTrans->scaleFactor.setValue(0.5, 0.45, 0.5);
  cubeTrans->rotation.setValue(0, 0, 1, 0.01);
  cubeSep->addChild(cubeTrans);

  SoMaterial *cubeMaterial = new SoMaterial();
  cubeMaterial->diffuseColor.setValue(1.0, 0.9, 0.2);
  cubeMaterial->shininess.setValue(0.0);
  cubeSep->addChild(cubeMaterial);
  
  SoCube *myCube = new SoCube();
  myCube->width.setValue(1.0);
  myCube->depth.setValue(1.0);
  myCube->height.setValue(1.0);
  cubeSep->addChild(myCube);

  // create sphere
  SoSeparator *sphereSep = new SoSeparator();
  root->addChild(sphereSep);

  SoTransform *sphereTrans = new SoTransform();
  sphereTrans->translation.setValue(-0.5, 0.5, -0.6);
  sphereSep->addChild(sphereTrans);

  SoMaterial *sphereMaterial = new SoMaterial();
  sphereMaterial->diffuseColor.setValue(0.4, 1.0, 0.4);
  sphereSep->addChild(sphereMaterial);

  SoSphere *mySphere = new SoSphere();
  mySphere->radius.setValue(0.4);
  sphereSep->addChild(mySphere);
  
  return root;
  }


void
animate(void *, SoSensor *)
  {
  //makeItMove ();
  PresentTime += DTime;
  }

void
main(int, char **argv)
  {
  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  SoSeparator *root = makeScene();

  SoTimerSensor *sensor = new SoTimerSensor(animate, root);
  sensor->setInterval(SbTime(0.015)); // 70 frames per second
  sensor->schedule();

  myViewer = new SoXtExaminerViewer(myWindow);
  myViewer->setTransparencyType(SoGLRenderAction::BLEND);
  myViewer->setBackgroundColor(SbColor(0.3, 0.3, 0.3));
  myViewer->setSceneGraph(root);
  myViewer->setTitle("Scene");
  myViewer->show();

  SoXt::show(myWindow);
  SoXt::mainLoop();
  }
